from typing import Any, List, Optional
import subprocess
from fastapi import UploadFile, File, Form
from fastapi.responses import FileResponse, JSONResponse
from fastapi import APIRouter, Depends, HTTPException
from fastapi import WebSocket, BackgroundTasks
from sqlalchemy.orm import Session
import asyncio
import time
import shutil
import app.crud as crud
import app.models as models
import app.schemas as schemas 
from app.api import deps
from datetime import datetime

from app.core.celery_app import celery_app
from app.core.config import settings
from pathlib import Path
from app.utils import random_name

from app.core.celery_app import celery_app

BACKEND_ZIP_STORAGE = Path("/app").joinpath(settings.BACKEND_ZIP_STORAGE)
LOCAL_ZIP_STORAGE = Path("/").joinpath(settings.LOCAL_ZIP_STORAGE)


router = APIRouter()


@router.post("/sr")
def supser_resolution(
    *,
    db: Session = Depends(deps.get_db),
    current_user: models.User = Depends(deps.get_current_active_user),
    upload_files: List[UploadFile]=File(description="Multiple files as UploadFile"),
    background_tasks: BackgroundTasks,
) -> Any:
    """
    Super Resolution.
    """
    print("api start: "+str(datetime.now()))
    filenames = [random_name(20)+Path(file.filename).suffix for file in upload_files]
    stemnames = [Path(filename).stem for filename in filenames]
    new_dir = random_name(10)
    new_dir_path = Path(BACKEND_ZIP_STORAGE).joinpath(new_dir)
    new_dir_path.mkdir()
    for i in range(len(upload_files)):
      try:
          with open(str(Path(new_dir_path).joinpath(filenames[i])), 'wb') as f:
              while contents := upload_files[i].file.read(1024 * 1024):
                  f.write(contents)
      except Exception as e:
          print(e, type(e))
          return {"error": str(e)}
      finally:
          upload_files[i].file.close()
    
    background_tasks.add_task(wait_finish, new_dir, filenames)

    print(filenames)
    return JSONResponse({"filenames": stemnames}, background=background_tasks)

async def wait_finish(dirname, filenames):
    # print("start: "+str(datetime.now()))
    new_dir_path = Path(BACKEND_ZIP_STORAGE).joinpath(dirname)
    process = await asyncio.create_subprocess_exec("sshpass", "-p", "choozmo9", 
                    "scp", "-P", "5722", "-o", "StrictHostKeyChecking=no", "-r", f"{str(new_dir_path)}", f"root@172.104.93.163:{str(LOCAL_ZIP_STORAGE)}")
    await process.wait()
    # r = subprocess.run(["sshpass", "-p", "choozmo9", 
    #                 "scp", "-P", "5722", "-o", "StrictHostKeyChecking=no", "-r", f"{str(new_dir_path)}", f"root@172.104.93.163:{str(LOCAL_ZIP_STORAGE)}"])
    
    restored_imgs = "restored_imgs" 
    res = celery_app.send_task("app.worker.super_resolution", args=[dirname])
    # print(res.state)
    while True:
        await asyncio.sleep(0.5)
        print(res.state)
        if res.status == 'SUCCESS':
            print("recieve finished")
            break
    for filename in filenames:
        process = await asyncio.create_subprocess_exec("sshpass", "-p", "choozmo9", 
                      "scp", "-P", "5722", "-o", "StrictHostKeyChecking=no", "-r", f"root@172.104.93.163:{str(LOCAL_ZIP_STORAGE/dirname/restored_imgs/filename)}", f"{str(BACKEND_ZIP_STORAGE)}")
        await process.wait()
        # re = subprocess.run(["sshpass", "-p", "choozmo9", 
        #             "scp", "-P", "5722", "-o", "StrictHostKeyChecking=no", "-r", f"root@172.104.93.163:{str(LOCAL_ZIP_STORAGE/dirname/restored_imgs/filename)}", f"{str(BACKEND_ZIP_STORAGE)}"])
        
        for sr_client in sr_clients.values():
            await sr_client.send_text(f"{filename}")
    # print("end: "+str(datetime.now()))

    

@router.get("/sr")
def get_image(
    *,
    db: Session = Depends(deps.get_db),
    current_user: models.User = Depends(deps.get_current_active_user),
    stored_file_name: str,
    file_name: str,
) -> Any:
    """
    Download image
    """
    
    return_file_path = list(BACKEND_ZIP_STORAGE.glob(stored_file_name+"*"))[0]
    return FileResponse(path=str(return_file_path), media_type='image/png', filename=file_name+return_file_path.suffix)

ws_clients = {}
sr_clients = {}

@router.websocket("/sr")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()
    key = websocket.headers.get('sec-websocket-key')
    sr_clients[key] = websocket
    print(f"new comer: {key}")
    try:
        while True:
            data = await websocket.receive_text()
            print(f"{key}:{data}")
            if data.startswith("unsubscribe"):
              del sr_clients[key]
              print(f"beybey: {key}")
              #for client in sr_clients.values():
              #      await client.send_text(f"ID: {key} | Message: {data}")

    except:
        #await websocket.close()
        # 接続が切れた場合、当該クライアントを削除する
        del sr_clients[key]
